"""
morinkit
~~~~~~~~

This package contains Python functions and script for genomics.
"""

__title__ = 'morinkit'
__version__ = '0.1.1'
__author__ = 'Morin Lab Members (Bruno Grande)'
__license__ = 'GNU GPLv3'
__url__ = 'https://github.com/morinlab/morinkit'
